@@ -49,5 +49,6 @@ enum options_client
 #ifdef HAVE_NDBCLUSTER_DB
   ,OPT_NDBCLUSTER,OPT_NDB_CONNECTSTRING
 #endif
-  ,OPT_IGNORE_TABLE,OPT_INSERT_IGNORE,OPT_DROP_DATABASE
+  ,OPT_IGNORE_TABLE,OPT_INSERT_IGNORE,OPT_DROP_DATABASE,
+  OPT_AUTO_CLOSE
 };
