@@ -485,7 +485,7 @@ que_graph_free_recursive(
 		if (thr->magic_n != QUE_THR_MAGIC_N) {
 			fprintf(stderr,
 		"que_thr struct appears corrupt; magic n %lu\n",
-								thr->magic_n);
+				(unsigned long) thr->magic_n);
 			mem_analyze_corruption((byte*)thr);
 			ut_error;
 		}
@@ -597,7 +597,7 @@ que_graph_free_recursive(
 	default:
 		fprintf(stderr,
 		"que_node struct appears corrupt; type %lu\n",
-						que_node_get_type(node));
+			(unsigned long) que_node_get_type(node));
 		mem_analyze_corruption((byte*)node);
 		ut_error;
 	}
@@ -986,7 +986,8 @@ que_thr_move_to_run_state_for_mysql(
 {
 	if (thr->magic_n != QUE_THR_MAGIC_N) {
 		fprintf(stderr,
-	"que_thr struct appears corrupt; magic n %lu\n", thr->magic_n);
+	"que_thr struct appears corrupt; magic n %lu\n",
+			(unsigned long) thr->magic_n);
 
 		mem_analyze_corruption((byte*)thr);
 
@@ -1022,7 +1023,8 @@ que_thr_stop_for_mysql_no_error(
 		
 	if (thr->magic_n != QUE_THR_MAGIC_N) {
 		fprintf(stderr,
-	"que_thr struct appears corrupt; magic n %lu\n", thr->magic_n);
+	"que_thr struct appears corrupt; magic n %lu\n",
+			(unsigned long) thr->magic_n);
 
 		mem_analyze_corruption((byte*)thr);
 
@@ -1094,7 +1096,7 @@ que_node_print_info(
 		str = "UNKNOWN NODE TYPE";
 	}
 
-	fprintf(stderr, "Node type %lu: %s, address %p\n", type, str, node);
+	fprintf(stderr, "Node type %lu: %s, address %p\n", (ulong) type, str, node);
 }
 
 /**************************************************************************
@@ -1253,10 +1255,6 @@ loop:
 		mutex_exit(&kernel_mutex);
 	}	
 */
-	/* TRUE below denotes that the thread is allowed to own the dictionary
-	mutex, though */
-	ut_ad(sync_thread_levels_empty_gen(TRUE));
-
 	loop_count++;
 
 	if (next_thr != thr) {
