@@ -462,6 +462,7 @@ int mysqld_extend_show_tables(THD *thd,const char *db,const char *wild)
   TABLE *table;
   Protocol *protocol= thd->protocol;
   TIME time;
+  int res= 0;
   DBUG_ENTER("mysqld_extend_show_tables");
 
   (void) sprintf(path,"%s/%s",mysql_data_home,db);
@@ -632,37 +633,15 @@ int mysqld_extend_show_tables(THD *thd,const char *db,const char *wild)
       close_thread_tables(thd,0);
     }
     if (protocol->write())
-      DBUG_RETURN(-1);
-  }
-  send_eof(thd);
-  DBUG_RETURN(0);
-}
-
-/*
-   returns the length of the longest type on the given table.
-
-   This is used so that show fields will return the data using the proper 
-   lengths instead of forcing columns such as type to always return with a 
-   given length.
-*/
-
-uint get_longest_type_in_table(TABLE *table, const char *wild)
-{  
-  Field **ptr,*field;
-  char tmp[MAX_FIELD_WIDTH];
-  uint max_len= 0;
-
-  for (ptr=table->field; (field= *ptr); ptr++)
-  {
-    if (!wild || !wild[0] ||
-        !wild_case_compare(system_charset_info, field->field_name,wild))
     {
-      String type(tmp,sizeof(tmp), system_charset_info);
-      field->sql_type(type);
-      max_len= max(max_len, type.length());
+      res= -1;
+      break;
     }
   }
-  return max_len;
+  thd->insert_id(0);
+  if (!res)
+    send_eof(thd);
+  DBUG_RETURN(res);
 }
 
 
@@ -694,14 +673,9 @@ mysqld_show_fields(THD *thd, TABLE_LIST *table_list,const char *wild,
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   (void) get_table_grant(thd, table_list);
 #endif
-
-  /* we scan for the longest since long enum types can exceed 40 */
-  uint max_len = get_longest_type_in_table(table, wild);
-
   List<Item> field_list;
   field_list.push_back(new Item_empty_string("Field",NAME_LEN));
-  field_list.push_back(new Item_empty_string("Type",
-	max_len > 40 ? max_len : 40));
+  field_list.push_back(new Item_empty_string("Type", 40));
   if (verbose)
     field_list.push_back(new Item_empty_string("Collation",40));
   field_list.push_back(new Item_empty_string("Null",1));
@@ -1331,7 +1305,8 @@ store_create_info(THD *thd, TABLE *table, String *packet)
     field->sql_type(type);
     packet->append(type.ptr(), type.length(), system_charset_info);
 
-    if (field->has_charset() && !limited_mysql_mode && !foreign_db_mode)
+    if (field->has_charset() && 
+        !(thd->variables.sql_mode & (MODE_MYSQL323 | MODE_MYSQL40)))
     {
       if (field->charset() != table->table_charset)
       {
@@ -1369,8 +1344,8 @@ store_create_info(THD *thd, TABLE *table, String *packet)
     
     has_default= (field->type() != FIELD_TYPE_BLOB &&
 		  field->unireg_check != Field::NEXT_NUMBER &&
-                  !((foreign_db_mode || limited_mysql_mode) &&
-                    has_now_default));
+                  !((thd->variables.sql_mode & (MODE_MYSQL323 | MODE_MYSQL40))
+		    && has_now_default));
 
     if (has_default)
     {
@@ -1399,12 +1374,12 @@ store_create_info(THD *thd, TABLE *table, String *packet)
         packet->append(tmp);
     }
 
-    if (!foreign_db_mode && !limited_mysql_mode &&
-        table->timestamp_field == field && 
+    if (!limited_mysql_mode && table->timestamp_field == field && 
         field->unireg_check != Field::TIMESTAMP_DN_FIELD)
       packet->append(" on update CURRENT_TIMESTAMP",28);
 
-    if (field->unireg_check == Field::NEXT_NUMBER && !foreign_db_mode)
+    if (field->unireg_check == Field::NEXT_NUMBER && 
+        !(thd->variables.sql_mode & MODE_NO_FIELD_OPTIONS))
       packet->append(" auto_increment", 15 );
 
     if (field->comment.length)
@@ -1504,6 +1479,24 @@ store_create_info(THD *thd, TABLE *table, String *packet)
       packet->append(" ENGINE=", 8);
     packet->append(file->table_type());
     
+    /*
+      Add AUTO_INCREMENT=... if there is an AUTO_INCREMENT column,
+      and NEXT_ID > 1 (the default).  We must not print the clause
+      for engines that do not support this as it would break the
+      import of dumps, but as of this writing, the test for whether
+      AUTO_INCREMENT columns are allowed and wether AUTO_INCREMENT=...
+      is supported is identical, !(file->table_flags() & HA_NO_AUTO_INCREMENT))
+      Because of that, we do not explicitly test for the feature,
+      but may extrapolate its existence from that of an AUTO_INCREMENT column.
+    */
+
+    if(create_info.auto_increment_value > 1)
+    {
+      packet->append(" AUTO_INCREMENT=", 16);
+      end= longlong10_to_str(create_info.auto_increment_value, buff,10);
+      packet->append(buff, (uint) (end - buff));
+    }
+    
     if (table->table_charset &&
 	!(thd->variables.sql_mode & MODE_MYSQL323) &&
 	!(thd->variables.sql_mode & MODE_MYSQL40))
@@ -1914,7 +1907,7 @@ int mysqld_show(THD *thd, const char *wild, show_var_st *variables,
       case SHOW_SLAVE_RUNNING:
       {
 	pthread_mutex_lock(&LOCK_active_mi);
-	end= strmov(buff, (active_mi->slave_running &&
+	end= strmov(buff, (active_mi && active_mi->slave_running &&
 			   active_mi->rli.slave_running) ? "ON" : "OFF");
 	pthread_mutex_unlock(&LOCK_active_mi);
 	break;
@@ -1926,9 +1919,12 @@ int mysqld_show(THD *thd, const char *wild, show_var_st *variables,
           SLAVE STATUS, and have the sum over all lines here.
         */
 	pthread_mutex_lock(&LOCK_active_mi);
-        pthread_mutex_lock(&active_mi->rli.data_lock);
-	end= int10_to_str(active_mi->rli.retried_trans, buff, 10);
-        pthread_mutex_unlock(&active_mi->rli.data_lock);
+        if (active_mi)
+        {
+          pthread_mutex_lock(&active_mi->rli.data_lock);
+          end= int10_to_str(active_mi->rli.retried_trans, buff, 10);
+          pthread_mutex_unlock(&active_mi->rli.data_lock);
+        }
 	pthread_mutex_unlock(&LOCK_active_mi);
 	break;
       }
@@ -2123,6 +2119,10 @@ int mysqld_show(THD *thd, const char *wild, show_var_st *variables,
 	value= (value-(char*) &dflt_key_cache_var)+ (char*) dflt_key_cache;
 	end= int10_to_str(*(long*) value, buff, 10);
         break;
+      case SHOW_KEY_CACHE_LONGLONG:
+	value= (value-(char*) &dflt_key_cache_var)+ (char*) dflt_key_cache;
+	end= longlong10_to_str(*(longlong*) value, buff, 10);
+	break;
       case SHOW_UNDEF:				// Show never happen
       case SHOW_SYS:
 	break;					// Return empty string
