@@ -18,41 +18,22 @@
 
 #include "ftdefs.h"
 
-#ifdef EVAL_RUN
-#ifdef PIVOT_STAT
-ulong collstat=0;
-#endif
-#endif /* EVAL_RUN */
-
 typedef struct st_ft_docstat {
   FT_WORD *list;
   uint uniq;
   double sum;
-#ifdef EVAL_RUN
-  uint words, totlen;
-  double max, nsum, nsum2;
-#endif /* EVAL_RUN */
-
 } FT_DOCSTAT;
 
 static int FT_WORD_cmp(CHARSET_INFO* cs, FT_WORD *w1, FT_WORD *w2)
 {
-  return _mi_compare_text(cs, (uchar*) w1->pos, w1->len,
-                              (uchar*) w2->pos, w2->len, 0);
+  return mi_compare_text(cs, (uchar*) w1->pos, w1->len,
+                         (uchar*) w2->pos, w2->len, 0, 0);
 }
 
 static int walk_and_copy(FT_WORD *word,uint32 count,FT_DOCSTAT *docstat)
 {
     word->weight=LWS_IN_USE;
-
-#ifdef EVAL_RUN
-    word->cnt= (uchar) count;
-    if(docstat->max < word->weight) docstat->max=word->weight;
-    docstat->words+=count;
-    docstat->totlen+=word->len;
-#endif /* EVAL_RUN */
     docstat->sum+=word->weight;
-
     memcpy_fixed((docstat->list)++,word,sizeof(FT_WORD));
     return 0;
 }
@@ -70,9 +51,6 @@ FT_WORD * ft_linearize(TREE *wtree)
   {
     docstat.list=wlist;
     docstat.uniq=wtree->elements_in_tree;
-#ifdef EVAL_RUN
-    docstat.nsum=docstat.nsum2=docstat.max=docstat.words=docstat.totlen=
-#endif /* EVAL_RUN */
     docstat.sum=0;
     tree_walk(wtree,(tree_walk_action)&walk_and_copy,&docstat,left_root_right);
   }
@@ -85,18 +63,8 @@ FT_WORD * ft_linearize(TREE *wtree)
   for (p=wlist;p->pos;p++)
   {
     p->weight=PRENORM_IN_USE;
-#ifdef EVAL_RUN
-    docstat.nsum+=p->weight;
-    docstat.nsum2+=p->weight*p->weight;
-#endif /* EVAL_RUN */
   }
 
-#ifdef EVAL_RUN
-#ifdef PIVOT_STAT
-  collstat+=PIVOT_STAT;
-#endif
-#endif /* EVAL_RUN */
-
   for (p=wlist;p->pos;p++)
   {
     p->weight/=NORM_IN_USE;
@@ -105,16 +73,37 @@ FT_WORD * ft_linearize(TREE *wtree)
   DBUG_RETURN(wlist);
 }
 
+my_bool ft_boolean_check_syntax_string(const byte *str)
+{
+  uint i, j;
+
+  if (!str ||
+      (strlen(str)+1 != sizeof(ft_boolean_syntax)) ||
+      (str[0] != ' ' && str[1] != ' '))
+    return 1;
+  for (i=0; i<sizeof(ft_boolean_syntax); i++)
+  {
+    /* limiting to 7-bit ascii only */
+    if ((unsigned char)(str[i]) > 127 || my_isalnum(default_charset_info, str[i]))
+      return 1;
+    for (j=0; j<i; j++)
+      if (str[i] == str[j] && (i != 11 || j != 10))
+        return 1;
+  }
+  return 0;
+}
+
 /* returns:
  * 0 - eof
  * 1 - word found
  * 2 - left bracket
  * 3 - right bracket
  */
-byte ft_get_word(byte **start, byte *end, FT_WORD *word, FTB_PARAM *param)
+byte ft_get_word(CHARSET_INFO *cs, byte **start, byte *end,
+                 FT_WORD *word, FTB_PARAM *param)
 {
   byte *doc=*start;
-  int mwc;
+  uint mwc, length, mbl;
 
   param->yesno=(FTB_YES==' ') ? 1 : (param->quot != 0);
   param->plusminus=param->pmsign=0;
@@ -123,7 +112,7 @@ byte ft_get_word(byte **start, byte *end, FT_WORD *word, FTB_PARAM *param)
   {
     for (;doc<end;doc++)
     {
-      if (true_word_char(*doc)) break;
+      if (true_word_char(cs,*doc)) break;
       if (*doc == FTB_RQUOT && param->quot)
       {
         param->quot=doc;
@@ -154,20 +143,22 @@ byte ft_get_word(byte **start, byte *end, FT_WORD *word, FTB_PARAM *param)
       param->plusminus=param->pmsign=0;
     }
 
-    mwc=0;
-    for (word->pos=doc; doc<end; doc++)
-      if (true_word_char(*doc))
+    mwc=length=0;
+    for (word->pos=doc; doc<end; length++, mbl=my_mbcharlen(cs, *(uchar *)doc), doc+=(mbl ? mbl : 1))
+      if (true_word_char(cs,*doc))
         mwc=0;
-      else if (!misc_word_char(*doc) || mwc++)
+      else if (!misc_word_char(*doc) || mwc)
         break;
+      else
+        mwc++;
 
     param->prev='A'; /* be sure *prev is true_word_char */
     word->len= (uint)(doc-word->pos) - mwc;
     if ((param->trunc=(doc<end && *doc == FTB_TRUNC)))
       doc++;
 
-    if (((word->len >= ft_min_word_len && !is_stopword(word->pos, word->len))
-         || param->trunc) && word->len < ft_max_word_len)
+    if (((length >= ft_min_word_len && !is_stopword(word->pos, word->len))
+         || param->trunc) && length < ft_max_word_len)
     {
       *start=doc;
       return 1;
@@ -181,32 +172,35 @@ byte ft_get_word(byte **start, byte *end, FT_WORD *word, FTB_PARAM *param)
   return 0;
 }
 
-byte ft_simple_get_word(byte **start, byte *end, FT_WORD *word)
+byte ft_simple_get_word(CHARSET_INFO *cs, byte **start, byte *end,
+                        FT_WORD *word)
 {
-  byte *doc=*start;
-  int mwc;
+  byte *doc= *start;
+  uint mwc, length, mbl;
   DBUG_ENTER("ft_simple_get_word");
 
   while (doc<end)
   {
     for (;doc<end;doc++)
     {
-      if (true_word_char(*doc)) break;
+      if (true_word_char(cs,*doc)) break;
     }
 
-    mwc=0;
-    for(word->pos=doc; doc<end; doc++)
-      if (true_word_char(*doc))
-        mwc=0;
-      else if (!misc_word_char(*doc) || mwc++)
+    mwc= length= 0;
+    for (word->pos=doc; doc<end; length++, mbl=my_mbcharlen(cs, *(uchar *)doc), doc+=(mbl ? mbl : 1))
+      if (true_word_char(cs,*doc))
+        mwc= 0;
+      else if (!misc_word_char(*doc) || mwc)
         break;
+      else
+        mwc++;
 
     word->len= (uint)(doc-word->pos) - mwc;
 
-    if (word->len >= ft_min_word_len && word->len < ft_max_word_len &&
+    if (length >= ft_min_word_len && length < ft_max_word_len &&
         !is_stopword(word->pos, word->len))
     {
-      *start=doc;
+      *start= doc;
       DBUG_RETURN(1);
     }
   }
@@ -221,15 +215,24 @@ void ft_parse_init(TREE *wtree, CHARSET_INFO *cs)
   DBUG_VOID_RETURN;
 }
 
-int ft_parse(TREE *wtree, byte *doc, int doclen)
+int ft_parse(TREE *wtree, byte *doc, int doclen, my_bool with_alloc)
 {
   byte   *end=doc+doclen;
   FT_WORD w;
   DBUG_ENTER("ft_parse");
 
-  while (ft_simple_get_word(&doc,end,&w))
+  while (ft_simple_get_word(wtree->custom_arg, &doc,end,&w))
   {
-    if (!tree_insert(wtree, &w, 0))
+    if (with_alloc)
+    {
+      byte *ptr;
+      /* allocating the data in the tree - to avoid mallocs and frees */
+      DBUG_ASSERT(wtree->with_delete==0);
+      ptr=(byte *)alloc_root(& wtree->mem_root,w.len);
+      memcpy(ptr, w.pos, w.len);
+      w.pos=ptr;
+    }
+    if (!tree_insert(wtree, &w, 0, wtree->custom_arg))
       goto err;
   }
   DBUG_RETURN(0);
