@@ -25,7 +25,6 @@
 */
 
 #include "myisamdef.h"
-#include <assert.h>
 
 /* Enough for comparing if number is zero */
 static char zero_string[]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
@@ -807,7 +806,7 @@ uint _mi_rec_pack(MI_INFO *info, register byte *to, register const byte *from)
 */
 
 my_bool _mi_rec_check(MI_INFO *info,const char *record, byte *rec_buff,
-                      ulong packed_length)
+                      ulong packed_length, my_bool with_checksum)
 {
   uint		length,new_length,flag,bit,i;
   char		*pos,*end,*packpos,*to;
@@ -902,13 +901,10 @@ my_bool _mi_rec_check(MI_INFO *info,const char *record, byte *rec_buff,
   if (packed_length != (uint) (to - rec_buff) + test(info->s->calc_checksum) ||
       (bit != 1 && (flag & ~(bit - 1))))
     goto err;
-  if (info->s->calc_checksum)
+  if (with_checksum && ((uchar) info->checksum != (uchar) *to))
   {
-    if ((uchar) info->checksum != (uchar) *to)
-    {
-      DBUG_PRINT("error",("wrong checksum for row"));
-      goto err;
-    }
+    DBUG_PRINT("error",("wrong checksum for row"));
+    goto err;
   }
   DBUG_RETURN(0);
 
@@ -1120,6 +1116,9 @@ int _mi_read_dynamic_record(MI_INFO *info, my_off_t filepos, byte *buf)
 	  info->rec_cache.pos_in_file <= block_info.next_filepos &&
 	  flush_io_cache(&info->rec_cache))
 	goto err;
+      /* A corrupted table can have wrong pointers. (Bug# 19835) */
+      if (block_info.next_filepos == HA_OFFSET_ERROR)
+        goto panic;
       info->rec_cache.seek_not_done=1;
       if ((b_type=_mi_get_block_info(&block_info,file,
 				     block_info.next_filepos))
@@ -1310,7 +1309,7 @@ err:
 
 int _mi_read_rnd_dynamic_record(MI_INFO *info, byte *buf,
 				register my_off_t filepos,
-				my_bool skipp_deleted_blocks)
+				my_bool skip_deleted_blocks)
 {
   int flag,info_read,save_errno;
   uint left_len,b_type;
@@ -1361,7 +1360,7 @@ int _mi_read_rnd_dynamic_record(MI_INFO *info, byte *buf,
     {
       if (_mi_read_cache(&info->rec_cache,(byte*) block_info.header,filepos,
 			 sizeof(block_info.header),
-			 (!flag && skipp_deleted_blocks ? READING_NEXT : 0) |
+			 (!flag && skip_deleted_blocks ? READING_NEXT : 0) |
 			 READING_HEADER))
 	goto panic;
       b_type=_mi_get_block_info(&block_info,-1,filepos);
@@ -1380,7 +1379,7 @@ int _mi_read_rnd_dynamic_record(MI_INFO *info, byte *buf,
 		  BLOCK_FATAL_ERROR))
     {
       if ((b_type & (BLOCK_DELETED | BLOCK_SYNC_ERROR))
-	  && skipp_deleted_blocks)
+	  && skip_deleted_blocks)
       {
 	filepos=block_info.filepos+block_info.block_len;
 	block_info.second_read=0;
@@ -1436,7 +1435,7 @@ int _mi_read_rnd_dynamic_record(MI_INFO *info, byte *buf,
       {
 	if (_mi_read_cache(&info->rec_cache,(byte*) to,filepos,
 			   block_info.data_len,
-			   (!flag && skipp_deleted_blocks) ? READING_NEXT :0))
+			   (!flag && skip_deleted_blocks) ? READING_NEXT :0))
 	  goto panic;
       }
       else
@@ -1453,7 +1452,7 @@ int _mi_read_rnd_dynamic_record(MI_INFO *info, byte *buf,
     if (flag++ == 0)
     {
       info->nextpos=block_info.filepos+block_info.block_len;
-      skipp_deleted_blocks=0;
+      skip_deleted_blocks=0;
     }
     left_len-=block_info.data_len;
     to+=block_info.data_len;
