@@ -39,13 +39,10 @@
 #define MYF_RW  MYF(MY_NABP | MY_WME | MY_WAIT_IF_FULL)
 #define DISK_BUFFER_SIZE (IO_SIZE*16)
 
-typedef struct st_buffpek {
-  my_off_t file_pos;                    /* Where we are in the sort file */
-  uchar *base,*key;                     /* Key pointers */
-  ha_rows count;                        /* Number of rows in table */
-  ulong mem_count;                      /* numbers of keys in memory */
-  ulong max_keys;                       /* Max keys in buffert */
-} BUFFPEK;
+
+/*
+ Pointers of functions for store and read keys from temp file
+*/
 
 extern void print_error _VARARGS((const char *fmt,...));
 
@@ -56,7 +53,7 @@ static ha_rows NEAR_F find_all_keys(MI_SORT_PARAM *info,uint keys,
                                     DYNAMIC_ARRAY *buffpek,int *maxbuffer,
                                     IO_CACHE *tempfile,
                                     IO_CACHE *tempfile_for_exceptions);
-static int NEAR_F write_keys(MI_SORT_PARAM *info,uchar * *sort_keys,
+static int NEAR_F write_keys(MI_SORT_PARAM *info,uchar **sort_keys,
                              uint count, BUFFPEK *buffpek,IO_CACHE *tempfile);
 static int NEAR_F write_key(MI_SORT_PARAM *info, uchar *key,
 			    IO_CACHE *tempfile);
@@ -74,7 +71,21 @@ static int NEAR_F merge_buffers(MI_SORT_PARAM *info,uint keys,
                                 BUFFPEK *Fb, BUFFPEK *Tb);
 static int NEAR_F merge_index(MI_SORT_PARAM *,uint,uchar **,BUFFPEK *, int,
                               IO_CACHE *);
+static int flush_ft_buf(MI_SORT_PARAM *info);
 
+static int NEAR_F write_keys_varlen(MI_SORT_PARAM *info,uchar **sort_keys,
+                                    uint count, BUFFPEK *buffpek,
+                                    IO_CACHE *tempfile);
+static uint NEAR_F read_to_buffer_varlen(IO_CACHE *fromfile,BUFFPEK *buffpek,
+                                         uint sort_length);
+static int NEAR_F write_merge_key(MI_SORT_PARAM *info, IO_CACHE *to_file,
+                                  char *key, uint sort_length, uint count);
+static int NEAR_F write_merge_key_varlen(MI_SORT_PARAM *info,
+					 IO_CACHE *to_file,
+					 char* key, uint sort_length,
+					 uint count);
+static inline int
+my_var_write(MI_SORT_PARAM *info, IO_CACHE *to_file, byte *bufs);
 
 /*
   Creates a index of sorted keys
@@ -102,6 +113,19 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
   DBUG_ENTER("_create_index_by_sort");
   DBUG_PRINT("enter",("sort_length: %d", info->key_length));
 
+  if (info->keyinfo->flag & HA_VAR_LENGTH_KEY)
+  {
+    info->write_keys=write_keys_varlen;
+    info->read_to_buffer=read_to_buffer_varlen;
+    info->write_key=write_merge_key_varlen;
+  }
+  else
+  {
+    info->write_keys=write_keys;
+    info->read_to_buffer=read_to_buffer;
+    info->write_key=write_merge_key;
+  }
+
   my_b_clear(&tempfile);
   my_b_clear(&tempfile_for_exceptions);
   bzero((char*) &buffpek,sizeof(buffpek));
@@ -134,11 +158,14 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
       while ((maxbuffer= (int) (records/(keys-1)+1)) != skr);
 
     if ((sort_keys=(uchar **)my_malloc(keys*(sort_length+sizeof(char*))+
-				       HA_FT_MAXLEN, MYF(0))))
+				       HA_FT_MAXBYTELEN, MYF(0))))
     {
       if (my_init_dynamic_array(&buffpek, sizeof(BUFFPEK), maxbuffer,
 			     maxbuffer/2))
+      {
 	my_free((gptr) sort_keys,MYF(0));
+        sort_keys= 0;
+      }
       else
 	break;
     }
@@ -182,13 +209,13 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
 	reinit_io_cache(&tempfile,READ_CACHE,0L,0,0))
       goto err;					/* purecov: inspected */
     if (!no_messages)
-      puts("  - Last merge and dumping keys\n"); /* purecov: tested */
+      printf("  - Last merge and dumping keys\n"); /* purecov: tested */
     if (merge_index(info,keys,sort_keys,dynamic_element(&buffpek,0,BUFFPEK *),
                     maxbuffer,&tempfile))
       goto err;					/* purecov: inspected */
   }
 
-  if (flush_pending_blocks(info))
+  if (flush_ft_buf(info) || flush_pending_blocks(info))
     goto err;
 
   if (my_b_inited(&tempfile_for_exceptions))
@@ -197,6 +224,8 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
     uint     keyno=info->key;
     uint     key_length, ref_length=index->s->rec_reflength;
 
+    if (!no_messages)
+      printf("  - Adding exceptions\n"); /* purecov: tested */
     if (flush_io_cache(&tempfile_for_exceptions) ||
 	reinit_io_cache(&tempfile_for_exceptions,READ_CACHE,0L,0,0))
       goto err;
@@ -249,7 +278,7 @@ static ha_rows NEAR_F find_all_keys(MI_SORT_PARAM *info, uint keys,
 
     if (++idx == keys)
     {
-      if (write_keys(info,sort_keys,idx-1,(BUFFPEK *)alloc_dynamic(buffpek),
+      if (info->write_keys(info,sort_keys,idx-1,(BUFFPEK *)alloc_dynamic(buffpek),
 		     tempfile))
       DBUG_RETURN(HA_POS_ERROR);		/* purecov: inspected */
 
@@ -263,7 +292,7 @@ static ha_rows NEAR_F find_all_keys(MI_SORT_PARAM *info, uint keys,
     DBUG_RETURN(HA_POS_ERROR);		/* Aborted by get_key */ /* purecov: inspected */
   if (buffpek->elements)
   {
-    if (write_keys(info,sort_keys,idx,(BUFFPEK *)alloc_dynamic(buffpek),
+    if (info->write_keys(info,sort_keys,idx,(BUFFPEK *)alloc_dynamic(buffpek),
 		   tempfile))
       DBUG_RETURN(HA_POS_ERROR);		/* purecov: inspected */
     *maxbuffer=buffpek->elements-1;
@@ -295,6 +324,19 @@ pthread_handler_decl(thr_find_all_keys,arg)
   if (info->sort_info->got_error)
     goto err;
 
+  if (info->keyinfo->flag && HA_VAR_LENGTH_KEY)
+  {
+    info->write_keys=write_keys_varlen;
+    info->read_to_buffer=read_to_buffer_varlen;
+    info->write_key=write_merge_key_varlen;
+  }
+  else
+  {
+    info->write_keys=write_keys;
+    info->read_to_buffer=read_to_buffer;
+    info->write_key=write_merge_key;
+  }
+
   my_b_clear(&info->tempfile);
   my_b_clear(&info->tempfile_for_exceptions);
   bzero((char*) &info->buffpek,sizeof(info->buffpek));
@@ -330,7 +372,7 @@ pthread_handler_decl(thr_find_all_keys,arg)
     }
     if ((sort_keys=(uchar **)my_malloc(keys*(sort_length+sizeof(char*))+
 				       ((info->keyinfo->flag & HA_FULLTEXT) ?
-					HA_FT_MAXLEN : 0), MYF(0))))
+					HA_FT_MAXBYTELEN : 0), MYF(0))))
     {
       if (my_init_dynamic_array(&info->buffpek, sizeof(BUFFPEK),
 				maxbuffer, maxbuffer/2))
@@ -367,7 +409,7 @@ pthread_handler_decl(thr_find_all_keys,arg)
 
     if (++idx == keys)
     {
-      if (write_keys(info,sort_keys,idx-1,
+      if (info->write_keys(info,sort_keys,idx-1,
 		     (BUFFPEK *)alloc_dynamic(&info->buffpek),
 		     &info->tempfile))
         goto err;
@@ -381,7 +423,7 @@ pthread_handler_decl(thr_find_all_keys,arg)
     goto err;
   if (info->buffpek.elements)
   {
-    if (write_keys(info,sort_keys, idx,
+    if (info->write_keys(info,sort_keys, idx,
 		   (BUFFPEK *) alloc_dynamic(&info->buffpek), &info->tempfile))
       goto err;
     info->keys=(info->buffpek.elements-1)*(keys-1)+idx;
@@ -439,8 +481,12 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
     {
       share->state.key_map|=(ulonglong) 1 << sinfo->key;
       if (param->testflag & T_STATISTICS)
-        update_key_parts(sinfo->keyinfo, rec_per_key_part,
-                         sinfo->unique, (ulonglong) info->state->records);
+        update_key_parts(sinfo->keyinfo, rec_per_key_part, sinfo->unique,
+                         param->stats_method == MI_STATS_METHOD_IGNORE_NULLS?
+                         sinfo->notnull: NULL,
+                         (ulonglong) info->state->records);
+
+
       if (!sinfo->buffpek.elements)
       {
         if (param->testflag & T_VERBOSE)
@@ -449,7 +495,7 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
           fflush(stdout);
         }
         if (write_index(sinfo, sinfo->sort_keys, sinfo->keys) ||
-            flush_pending_blocks(sinfo))
+            flush_ft_buf(sinfo) || flush_pending_blocks(sinfo))
           got_error=1;
       }
     }
@@ -469,6 +515,18 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
   {
     if (got_error)
       continue;
+    if (sinfo->keyinfo->flag && HA_VAR_LENGTH_KEY)
+    {
+      sinfo->write_keys=write_keys_varlen;
+      sinfo->read_to_buffer=read_to_buffer_varlen;
+      sinfo->write_key=write_merge_key_varlen;
+    }
+    else
+    {
+      sinfo->write_keys=write_keys;
+      sinfo->read_to_buffer=read_to_buffer;
+      sinfo->write_key=write_merge_key;
+    }
     if (sinfo->buffpek.elements)
     {
       uint maxbuffer=sinfo->buffpek.elements-1;
@@ -510,6 +568,7 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
       if (merge_index(sinfo, keys, (uchar **)mergebuf,
                       dynamic_element(&sinfo->buffpek,0,BUFFPEK *),
                       maxbuffer,&sinfo->tempfile) ||
+          flush_ft_buf(sinfo) ||
 	  flush_pending_blocks(sinfo))
       {
         got_error=1;
@@ -534,7 +593,7 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
 	     !my_b_read(&sinfo->tempfile_for_exceptions,(byte*)&key_length,
 			sizeof(key_length)))
       {
-        byte ft_buf[HA_FT_MAXLEN + HA_FT_WLEN + 10];
+        byte ft_buf[HA_FT_MAXBYTELEN + HA_FT_WLEN + 10];
         if (key_length > sizeof(ft_buf) ||
             my_b_read(&sinfo->tempfile_for_exceptions, (byte*)ft_buf,
                       (uint)key_length) ||
@@ -561,8 +620,8 @@ static int NEAR_F write_keys(MI_SORT_PARAM *info, register uchar **sort_keys,
   qsort2((byte*) sort_keys,count,sizeof(byte*),(qsort2_cmp) info->key_cmp,
          info);
   if (!my_b_inited(tempfile) &&
-      open_cached_file(tempfile, info->tmpdir, "ST", DISK_BUFFER_SIZE,
-                       info->sort_info->param->myf_rw))
+      open_cached_file(tempfile, my_tmpdir(info->tmpdir), "ST",
+                       DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
     DBUG_RETURN(1); /* purecov: inspected */
 
   buffpek->file_pos=my_b_tell(tempfile);
@@ -577,6 +636,48 @@ static int NEAR_F write_keys(MI_SORT_PARAM *info, register uchar **sort_keys,
 } /* write_keys */
 
 
+static inline int
+my_var_write(MI_SORT_PARAM *info, IO_CACHE *to_file, byte *bufs)
+{
+  int err;
+  uint16 len = _mi_keylength(info->keyinfo, (uchar*) bufs);
+
+  /* The following is safe as this is a local file */
+  if ((err= my_b_write(to_file, (byte*)&len, sizeof(len))))
+    return (err);
+  if ((err= my_b_write(to_file,bufs, (uint) len)))
+    return (err);
+  return (0);
+}
+
+
+static int NEAR_F write_keys_varlen(MI_SORT_PARAM *info,
+				    register uchar **sort_keys,
+                                    uint count, BUFFPEK *buffpek,
+				    IO_CACHE *tempfile)
+{
+  uchar **end;
+  int err;
+  DBUG_ENTER("write_keys_varlen");
+
+  qsort2((byte*) sort_keys,count,sizeof(byte*),(qsort2_cmp) info->key_cmp,
+         info);
+  if (!my_b_inited(tempfile) &&
+      open_cached_file(tempfile, my_tmpdir(info->tmpdir), "ST",
+                       DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
+    DBUG_RETURN(1); /* purecov: inspected */
+
+  buffpek->file_pos=my_b_tell(tempfile);
+  buffpek->count=count;
+  for (end=sort_keys+count ; sort_keys != end ; sort_keys++)
+  {
+    if ((err= my_var_write(info,tempfile, (byte*) *sort_keys)))
+      DBUG_RETURN(err);
+  }
+  DBUG_RETURN(0);
+} /* write_keys_varlen */
+
+
 static int NEAR_F write_key(MI_SORT_PARAM *info, uchar *key,
 			    IO_CACHE *tempfile)
 {
@@ -584,8 +685,8 @@ static int NEAR_F write_key(MI_SORT_PARAM *info, uchar *key,
   DBUG_ENTER("write_key");
 
   if (!my_b_inited(tempfile) &&
-      open_cached_file(tempfile, info->tmpdir, "ST", DISK_BUFFER_SIZE,
-                       info->sort_info->param->myf_rw))
+      open_cached_file(tempfile, my_tmpdir(info->tmpdir), "ST",
+                       DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
     DBUG_RETURN(1);
 
   if (my_b_write(tempfile,(byte*)&key_length,sizeof(key_length)) ||
@@ -627,8 +728,8 @@ static int NEAR_F merge_many_buff(MI_SORT_PARAM *info, uint keys,
   if (*maxbuffer < MERGEBUFF2)
     DBUG_RETURN(0);                             /* purecov: inspected */
   if (flush_io_cache(t_file) ||
-      open_cached_file(&t_file2,info->tmpdir,"ST",DISK_BUFFER_SIZE,
-                       info->sort_info->param->myf_rw))
+      open_cached_file(&t_file2,my_tmpdir(info->tmpdir),"ST",
+                       DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
     DBUG_RETURN(1);                             /* purecov: inspected */
 
   from_file= t_file ; to_file= &t_file2;
@@ -691,6 +792,62 @@ static uint NEAR_F read_to_buffer(IO_CACHE *fromfile, BUFFPEK *buffpek,
   return (count*sort_length);
 } /* read_to_buffer */
 
+static uint NEAR_F read_to_buffer_varlen(IO_CACHE *fromfile, BUFFPEK *buffpek,
+                                         uint sort_length)
+{
+  register uint count;
+  uint16 length_of_key = 0;
+  uint idx;
+  uchar *buffp;
+
+  if ((count=(uint) min((ha_rows) buffpek->max_keys,buffpek->count)))
+  {
+    buffp = buffpek->base;
+
+    for (idx=1;idx<=count;idx++)
+    {
+      if (my_pread(fromfile->file,(byte*)&length_of_key,sizeof(length_of_key),
+                   buffpek->file_pos,MYF_RW))
+        return((uint) -1);
+      buffpek->file_pos+=sizeof(length_of_key);
+      if (my_pread(fromfile->file,(byte*) buffp,length_of_key,
+                   buffpek->file_pos,MYF_RW))
+        return((uint) -1);
+      buffpek->file_pos+=length_of_key;
+      buffp = buffp + sort_length;
+    }
+    buffpek->key=buffpek->base;
+    buffpek->count-=    count;
+    buffpek->mem_count= count;
+  }
+  return (count*sort_length);
+} /* read_to_buffer_varlen */
+
+
+static int NEAR_F write_merge_key_varlen(MI_SORT_PARAM *info,
+					 IO_CACHE *to_file,char* key,
+                                         uint sort_length, uint count)
+{
+  uint idx;
+
+  char *bufs = key;
+  for (idx=1;idx<=count;idx++)
+  {
+    int err;
+    if ((err= my_var_write(info,to_file, (byte*) bufs)))
+      return (err);
+    bufs=bufs+sort_length;
+  }
+  return(0);
+}
+
+
+static int NEAR_F write_merge_key(MI_SORT_PARAM *info __attribute__((unused)),
+				  IO_CACHE *to_file, char* key,
+				  uint sort_length, uint count)
+{
+  return my_b_write(to_file,(byte*) key,(uint) sort_length*count);
+}
 
 /*
   Merge buffers to one buffer
@@ -709,6 +866,7 @@ merge_buffers(MI_SORT_PARAM *info, uint keys, IO_CACHE *from_file,
   uchar *strpos;
   BUFFPEK *buffpek,**refpek;
   QUEUE queue;
+  volatile my_bool *killed= killed_ptr(info->sort_info->param);
   DBUG_ENTER("merge_buffers");
 
   count=error=0;
@@ -729,8 +887,8 @@ merge_buffers(MI_SORT_PARAM *info, uint keys, IO_CACHE *from_file,
     count+= buffpek->count;
     buffpek->base= strpos;
     buffpek->max_keys=maxcount;
-    strpos+= (uint) (error=(int) read_to_buffer(from_file,buffpek,
-                                                sort_length));
+    strpos+= (uint) (error=(int) info->read_to_buffer(from_file,buffpek,
+                                                      sort_length));
     if (error == -1)
       goto err; /* purecov: inspected */
     queue_insert(&queue,(char*) buffpek);
@@ -740,10 +898,15 @@ merge_buffers(MI_SORT_PARAM *info, uint keys, IO_CACHE *from_file,
   {
     for (;;)
     {
+      if (*killed)
+      {
+        error=1; goto err;
+      }
       buffpek=(BUFFPEK*) queue_top(&queue);
       if (to_file)
       {
-        if (my_b_write(to_file,(byte*) buffpek->key,(uint) sort_length))
+        if (info->write_key(info,to_file,(byte*) buffpek->key,
+                            (uint) sort_length,1))
         {
           error=1; goto err; /* purecov: inspected */
         }
@@ -758,7 +921,7 @@ merge_buffers(MI_SORT_PARAM *info, uint keys, IO_CACHE *from_file,
       buffpek->key+=sort_length;
       if (! --buffpek->mem_count)
       {
-        if (!(error=(int) read_to_buffer(from_file,buffpek,sort_length)))
+        if (!(error=(int) info->read_to_buffer(from_file,buffpek,sort_length)))
         {
           uchar *base=buffpek->base;
           uint max_keys=buffpek->max_keys;
@@ -798,8 +961,8 @@ merge_buffers(MI_SORT_PARAM *info, uint keys, IO_CACHE *from_file,
   {
     if (to_file)
     {
-      if (my_b_write(to_file,(byte*) buffpek->key,
-                     (sort_length*buffpek->mem_count)))
+      if (info->write_key(info,to_file,(byte*) buffpek->key,
+                         sort_length,buffpek->mem_count))
       {
         error=1; goto err; /* purecov: inspected */
       }
@@ -819,7 +982,7 @@ merge_buffers(MI_SORT_PARAM *info, uint keys, IO_CACHE *from_file,
       }
     }
   }
-  while ((error=(int) read_to_buffer(from_file,buffpek,sort_length)) != -1 &&
+  while ((error=(int) info->read_to_buffer(from_file,buffpek,sort_length)) != -1 &&
          error != 0);
 
   lastbuff->count=count;
@@ -844,3 +1007,16 @@ merge_index(MI_SORT_PARAM *info, uint keys, uchar **sort_keys,
   DBUG_RETURN(0);
 } /* merge_index */
 
+static int
+flush_ft_buf(MI_SORT_PARAM *info)
+{
+  int err=0;
+  if (info->sort_info->ft_buf)
+  {
+    err=sort_ft_buf_flush(info);
+    my_free((gptr)info->sort_info->ft_buf, MYF(0));
+    info->sort_info->ft_buf=0;
+  }
+  return err;
+}
+
