@@ -18,10 +18,10 @@
 #include "mysys_priv.h"
 #include <sys/stat.h>
 #include <m_string.h>
-#if defined(HAVE_SYS_UTIME_H)
-#include <sys/utime.h>
-#elif defined(HAVE_UTIME_H)
+#if defined(HAVE_UTIME_H)
 #include <utime.h>
+#elif defined(HAVE_SYS_UTIME_H)
+#include <sys/utime.h>
 #elif !defined(HPUX10)
 #include <time.h>
 struct utimbuf {
