@@ -25,7 +25,6 @@
 #include <AttributeHeader.hpp>
 #include <Interpreter.hpp>
 #include <signaldata/FsConf.hpp>
-#include <signaldata/FsRef.hpp>
 #include <signaldata/FsRemoveReq.hpp>
 #include <signaldata/TupCommit.hpp>
 #include <signaldata/TupKey.hpp>
@@ -117,13 +116,9 @@ Dbtup::Dbtup(const class Configuration & conf)
   addRecSignal(GSN_TUP_SRREQ, &Dbtup::execTUP_SRREQ);
   addRecSignal(GSN_TUP_PREPLCPREQ, &Dbtup::execTUP_PREPLCPREQ);
   addRecSignal(GSN_FSOPENCONF, &Dbtup::execFSOPENCONF);
-  addRecSignal(GSN_FSOPENREF, &Dbtup::execFSOPENREF);
   addRecSignal(GSN_FSCLOSECONF, &Dbtup::execFSCLOSECONF);
-  addRecSignal(GSN_FSCLOSEREF, &Dbtup::execFSCLOSEREF);
   addRecSignal(GSN_FSWRITECONF, &Dbtup::execFSWRITECONF);
-  addRecSignal(GSN_FSWRITEREF, &Dbtup::execFSWRITEREF);
   addRecSignal(GSN_FSREADCONF, &Dbtup::execFSREADCONF);
-  addRecSignal(GSN_FSREADREF, &Dbtup::execFSREADREF);
   addRecSignal(GSN_NDB_STTOR, &Dbtup::execNDB_STTOR);
   addRecSignal(GSN_READ_CONFIG_REQ, &Dbtup::execREAD_CONFIG_REQ, true);
   addRecSignal(GSN_SET_VAR_REQ,  &Dbtup::execSET_VAR_REQ);
@@ -133,7 +128,6 @@ Dbtup::Dbtup(const class Configuration & conf)
   addRecSignal(GSN_DROP_TRIG_REQ,  &Dbtup::execDROP_TRIG_REQ);
 
   addRecSignal(GSN_DROP_TAB_REQ, &Dbtup::execDROP_TAB_REQ);
-  addRecSignal(GSN_FSREMOVEREF, &Dbtup::execFSREMOVEREF);
   addRecSignal(GSN_FSREMOVECONF, &Dbtup::execFSREMOVECONF);
 
   addRecSignal(GSN_TUP_ALLOCREQ, &Dbtup::execTUP_ALLOCREQ);
@@ -263,12 +257,6 @@ void Dbtup::execFSCLOSECONF(Signal* signal)
   releasePendingFileOpenInfoRecord(pfoPtr);
 }//Dbtup::execFSCLOSECONF()
 
-void Dbtup::execFSCLOSEREF(Signal* signal) 
-{
-  ljamEntry();
-  ndbrequire(false);
-}//Dbtup::execFSCLOSEREF()
-
 void Dbtup::execFSOPENCONF(Signal* signal) 
 {
   PendingFileOpenInfoPtr pfoPtr;
@@ -363,12 +351,6 @@ void Dbtup::execFSOPENCONF(Signal* signal)
   releasePendingFileOpenInfoRecord(pfoPtr);
 }//Dbtup::execFSOPENCONF()
 
-void Dbtup::execFSOPENREF(Signal* signal) 
-{
-  ljamEntry();
-  ndbrequire(false);
-}//Dbtup::execFSOPENREF()
-
 void Dbtup::execFSREADCONF(Signal* signal) 
 {
   DiskBufferSegmentInfoPtr dbsiPtr;
@@ -424,12 +406,6 @@ void Dbtup::execFSREADCONF(Signal* signal)
   }//switch
 }//Dbtup::execFSREADCONF()
 
-void Dbtup::execFSREADREF(Signal* signal) 
-{
-  ljamEntry();
-  ndbrequire(false);
-}//Dbtup::execFSREADREF()
-
 void Dbtup::execFSWRITECONF(Signal* signal) 
 {
   DiskBufferSegmentInfoPtr dbsiPtr;
@@ -478,12 +454,6 @@ void Dbtup::execFSWRITECONF(Signal* signal)
   return;
 }//Dbtup::execFSWRITECONF()
 
-void Dbtup::execFSWRITEREF(Signal* signal) 
-{
-  ljamEntry();
-  ndbrequire(false);
-}//Dbtup::execFSWRITEREF()
-
 void Dbtup::execCONTINUEB(Signal* signal) 
 {
   ljamEntry();
