@@ -138,7 +138,10 @@ public:
   {
     return (null_value=args[0]->get_time(ltime));
   }
-  bool is_null() { (void) val_int(); return null_value; }
+  bool is_null() { 
+    (void) val_int();  /* Discard result. It sets null_value as side-effect. */ 
+    return null_value; 
+  }
   friend class udf_handler;
   Field *tmp_table_field() { return result_field; }
   Field *tmp_table_field(TABLE *t_arg);
@@ -938,7 +941,6 @@ class user_var_entry;
 class Item_func_set_user_var :public Item_func
 {
   enum Item_result cached_result_type;
-  LEX_STRING name;
   user_var_entry *entry;
   char buffer[MAX_FIELD_WIDTH];
   String value;
@@ -952,6 +954,7 @@ class Item_func_set_user_var :public Item_func
   
 
 public:
+  LEX_STRING name; // keep it public
   Item_func_set_user_var(LEX_STRING a,Item *b)
     :Item_func(b), cached_result_type(INT_RESULT), name(a)
   {}
@@ -972,10 +975,10 @@ public:
 
 class Item_func_get_user_var :public Item_func
 {
-  LEX_STRING name;
   user_var_entry *var_entry;
 
 public:
+  LEX_STRING name; // keep it public
   Item_func_get_user_var(LEX_STRING a):
     Item_func(), name(a) {}
   double val();
@@ -1056,16 +1059,13 @@ public:
     if (!master && ft_handler)
     {
       ft_handler->please->close_search(ft_handler);
-      ft_handler=0;
-      if (join_key)
-	table->file->ft_handler=0;
-      table->fulltext_searched=0;
     }
     if (concat)
     {
       delete concat;
       concat= 0;
     }
+    ft_handler= 0;
     DBUG_VOID_RETURN;
   }
   enum Functype functype() const { return FT_FUNC; }
