@@ -17,7 +17,7 @@
 
 /* This file defines structures needed by udf functions */
 
-#ifdef __GNUC__
+#ifdef USE_PRAGMA_INTERFACE
 #pragma interface
 #endif
 
@@ -25,8 +25,7 @@ enum Item_udftype {UDFTYPE_FUNCTION=1,UDFTYPE_AGGREGATE};
 
 typedef struct st_udf_func
 {
-  char *name;
-  int name_length;
+  LEX_STRING name;
   Item_result returns;
   Item_udftype type;
   char *dl;
@@ -34,7 +33,7 @@ typedef struct st_udf_func
   void *func;
   void *func_init;
   void *func_deinit;
-  void *func_reset;
+  void *func_clear;
   void *func_add;
   ulong usage_count;
 } udf_func;
@@ -50,23 +49,25 @@ class udf_handler :public Sql_alloc
   UDF_ARGS f_args;
   UDF_INIT initid;
   char *num_buffer;
-  uchar error;
+  uchar error, is_null;
   bool initialized;
   Item **args;
 
  public:
   table_map used_tables_cache;
   bool const_item_cache;
+  bool not_original;
   udf_handler(udf_func *udf_arg) :u_d(udf_arg), buffers(0), error(0),
-    initialized(0)
+    is_null(0), initialized(0), not_original(0)
   {}
   ~udf_handler();
-  const char *name() const { return u_d ? u_d->name : "?"; }
+  const char *name() const { return u_d ? u_d->name.str : "?"; }
   Item_result result_type () const
   { return u_d	? u_d->returns : STRING_RESULT;}
   bool get_arguments();
   bool fix_fields(THD *thd,struct st_table_list *tlist,Item_result_field *item,
 		  uint arg_count,Item **args);
+  void cleanup();
   double val(my_bool *null_value)
   {
     if (get_arguments())
@@ -74,7 +75,6 @@ class udf_handler :public Sql_alloc
       *null_value=1;
       return 0.0;
     }
-    uchar is_null=0;
     double (*func)(UDF_INIT *, UDF_ARGS *, uchar *, uchar *)=
       (double (*)(UDF_INIT *, UDF_ARGS *, uchar *, uchar *)) u_d->func;
     double tmp=func(&initid, &f_args, &is_null, &error);
@@ -93,7 +93,6 @@ class udf_handler :public Sql_alloc
       *null_value=1;
       return LL(0);
     }
-    uchar is_null=0;
     longlong (*func)(UDF_INIT *, UDF_ARGS *, uchar *, uchar *)=
       (longlong (*)(UDF_INIT *, UDF_ARGS *, uchar *, uchar *)) u_d->func;
     longlong tmp=func(&initid, &f_args, &is_null, &error);
@@ -105,22 +104,15 @@ class udf_handler :public Sql_alloc
     *null_value=0;
     return tmp;
   }
-  void reset(my_bool *null_value)
+  void clear()
   {
-    uchar is_null=0;
-    if (get_arguments())
-    {
-      *null_value=1;
-      return;
-    }
-    void (*func)(UDF_INIT *, UDF_ARGS *, uchar *, uchar *)=
-    (void (*)(UDF_INIT *, UDF_ARGS *, uchar *, uchar *)) u_d->func_reset;
-    func(&initid, &f_args, &is_null, &error);
-    *null_value= (my_bool) (is_null || error);
+    is_null= 0;
+    void (*func)(UDF_INIT *, uchar *, uchar *)=
+    (void (*)(UDF_INIT *, uchar *, uchar *)) u_d->func_clear;
+    func(&initid, &is_null, &error);
   }
   void add(my_bool *null_value)
   {
-    uchar is_null=0;
     if (get_arguments())
     {
       *null_value=1;
@@ -140,5 +132,5 @@ void udf_init(void),udf_free(void);
 udf_func *find_udf(const char *name, uint len=0,bool mark_used=0);
 void free_udf(udf_func *udf);
 int mysql_create_function(THD *thd,udf_func *udf);
-int mysql_drop_function(THD *thd,const char *name);
+int mysql_drop_function(THD *thd,const LEX_STRING *name);
 #endif
